/**
 * Vue 插件：开启关闭rem支持
 * @author linyupark@gmail.com
 */

const dpr = window.devicePixelRatio || 1;
const docEl = document.documentElement;

export default {
  /**
   * 安装Vue插件
   * @param {Vue} Vue
   * @param {object} options 插件引入的参数对象
   */
  install(Vue, options={}) {

    let _flag = false;

    const pxToRem = options.pxToRem || 75;

    const setRemUnit = function() {
      const rem =
        docEl.clientWidth <= (pxToRem * 10) ? docEl.clientWidth / 10 : pxToRem;
      docEl.style.fontSize = rem + 'px';
    };

    const pageShow = function(event) {
      if (event.persisted) {
        setTimeout(() => {
          setRemUnit();
        }, 10);
      }
    };

    const setBodyFontSize = function() {
      if (document.body) {
        document.body.style.fontSize =
          docEl.clientWidth <= (pxToRem * 10) ? 12 * dpr + 'px' : pxToRem + 'px';
      } else {
        document.addEventListener(
          'DOMContentLoaded',
          setBodyFontSize,
          false
        );
      }
    };

    Vue.mixin({
      beforeCreate() {
        if(!_flag) this.$rem(true);
      }
    });

    Vue.prototype.$rem = function(flag) {
      if(flag) {
        console.log('RemPlugin ... on');
        setBodyFontSize();
        setRemUnit();
        if (dpr >= 2) {
          let fakeBody = document.createElement('body');
          let testElement = document.createElement('div');
          testElement.style.border = '.5px solid transparent';
          fakeBody.appendChild(testElement);
          docEl.appendChild(fakeBody);
          if (testElement.offsetHeight === 1) {
            docEl.classList.add('hairlines');
          }
          docEl.removeChild(fakeBody);
        }
        window.addEventListener('resize', setRemUnit, false);
        window.addEventListener('pageShow', pageShow, false);
        _flag = true;
      }
      else {
        console.log('RemPlugin ... off');
        document.body.style.fontSize = null;
        docEl.style.fontSize = null;
        document.removeEventListener('DOMContentLoaded', setBodyFontSize, false);
        window.removeEventListener('resize', setRemUnit, false);
        window.removeEventListener('pageShow', pageShow, false);
        _flag = false;
      }
    };
  }
};
